'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generatePatchesForMainForEject = exports.transformMainForEject = undefined;

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _diff = require('diff');

var _core = require('jscodeshift/dist/core');

var _core2 = _interopRequireDefault(_core);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function generatePatchesForMainForEject(mainPath, mainSource, transformedSource) {

  var patch = (0, _diff.createPatch)(mainPath, mainSource, transformedSource);

  var coloredPatch = '';

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(patch.split('\n')), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var l = _step.value;

      if (l.startsWith('-')) {
        coloredPatch += _chalk2.default.red(l) + '\n';
      } else if (l.startsWith('+')) {
        coloredPatch += _chalk2.default.green(l) + '\n';
      } else {
        coloredPatch += l + '\n';
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return coloredPatch;
}
// NOTE: this is an undocumented API and liable to break after upgrading jscodeshift


function transformMainForEject(mainSource, moduleName) {
  var ast = (0, _core2.default)(mainSource);

  var foundReactNativeImport = false;
  var rootClassId = null;

  // remove the Exponent.registerRootComponent call, and also record the name of the root component
  ast.find(_core2.default.ExpressionStatement).filter(function (e) {
    return e.value.expression.callee.object.name === 'Exponent';
  }).filter(function (e) {
    return e.value.expression.callee.property.name === 'registerRootComponent';
  }).forEach(function (e) {
    rootClassId = _core2.default.identifier(e.value.expression.arguments[0].name);
    e.prune();
  });

  // add the vanilla react native appregistry command
  if (!rootClassId) {
    throw new Error('Unable to find an invocation of `Exponent.registerRootComponent` to transform.');
  }

  var reactNativeAppRegistryInvocation = _core2.default.expressionStatement(_core2.default.callExpression(_core2.default.memberExpression(_core2.default.identifier('AppRegistry'), _core2.default.identifier('registerComponent')), [_core2.default.literal(moduleName), _core2.default.arrowFunctionExpression([], rootClassId)]));

  ast.find(_core2.default.Program).forEach(function (p) {
    return p.value.body.push(reactNativeAppRegistryInvocation);
  });

  // add appregistry to existing react-native imports if they exist (how could they not?)
  var appRegistryImport = _core2.default.importSpecifier(_core2.default.identifier('AppRegistry'));
  ast.find(_core2.default.ImportDeclaration).filter(function (i) {
    return i.value.source.value === 'react-native';
  }).forEach(function (i) {
    // TODO check to see if appregistry is in the imports before doing any of this
    i.value.specifiers.push(appRegistryImport);

    // people might gripe about sorting these, but in dikaiosune's opinion it's better than
    // sticking a guaranteed-out-of-order import at the end
    i.value.specifiers.sort(function (a, b) {
      return a.imported.name > b.imported.name;
    });

    foundReactNativeImport = true;
  });

  // handle case where there's no react-native imports
  if (!foundReactNativeImport) {
    (function () {
      // lol how did you even build this app?
      var newImport = _core2.default.importDeclaration([appRegistryImport], _core2.default.literal('react-native'));
      ast.find(_core2.default.Program).forEach(function (p) {
        return p.value.body.splice(0, 0, newImport);
      });
    })();
  }

  // clear exponent import
  ast.find(_core2.default.ImportDeclaration).filter(function (i) {
    return i.value.source.value === 'exponent';
  }).forEach(function (i) {
    return i.prune();
  });

  return ast.toSource({
    quote: 'single'
  });
}

exports.transformMainForEject = transformMainForEject;
exports.generatePatchesForMainForEject = generatePatchesForMainForEject;
//# sourceMappingURL=../__sourcemaps__/util/xforms.js.map
